`timescale 1ns/10ps
`celldefine
module ADDFX1 (A, B, CI, CO, S);
input  A ;
input  B ;
input  CI ;
output CO ;
output S ;

   xor (I0_out, A, B);
   xor (S, I0_out, CI);
   and (I2_out, A, B);
   and (I3_out, B, CI);
   and (I5_out, CI, A);
   or  (CO, I2_out, I3_out, I5_out);

   specify
     // delay parameters
     specparam
       tpllh$B$S = 0.22:0.23:0.24,
       tplhl$B$S = 0.2:0.21:0.21,
       tpllh$B$CO = 0.12:0.12:0.12,
       tphhl$B$CO = 0.14:0.14:0.14,
       tpllh$A$S = 0.23:0.23:0.24,
       tplhl$A$S = 0.19:0.2:0.22,
       tpllh$A$CO = 0.1:0.11:0.13,
       tphhl$A$CO = 0.13:0.13:0.13,
       tpllh$CI$S = 0.2:0.21:0.22,
       tplhl$CI$S = 0.19:0.19:0.2,
       tpllh$CI$CO = 0.1:0.11:0.12,
       tphhl$CI$CO = 0.12:0.12:0.12;

     // path delays
     (A *> CO) = (tpllh$A$CO, tphhl$A$CO);
     (A *> S) = (tpllh$A$S, tplhl$A$S);
     (B *> CO) = (tpllh$B$CO, tphhl$B$CO);
     (B *> S) = (tpllh$B$S, tplhl$B$S);
     (CI *> CO) = (tpllh$CI$CO, tphhl$CI$CO);
     (CI *> S) = (tpllh$CI$S, tplhl$CI$S);

   endspecify

endmodule
`endcelldefine

`timescale 1ns/10ps
`celldefine
module AND2X1 (A, B, Y);
input  A ;
input  B ;
output Y ;

   and (Y, A, B);

   specify
     // delay parameters
     specparam
       tpllh$A$Y = 0.061:0.061:0.061,
       tphhl$A$Y = 0.05:0.05:0.05,
       tpllh$B$Y = 0.063:0.063:0.063,
       tphhl$B$Y = 0.055:0.055:0.055;

     // path delays
     (A *> Y) = (tpllh$A$Y, tphhl$A$Y);
     (B *> Y) = (tpllh$B$Y, tphhl$B$Y);

   endspecify

endmodule
`endcelldefine

`timescale 1ns/10ps
`celldefine
module AOI21X1 (A0, A1, B0, Y);
input  A0 ;
input  A1 ;
input  B0 ;
output Y ;

   and (I0_out, A0, A1);
   or  (I1_out, I0_out, B0);
   not (Y, I1_out);

   specify
     // delay parameters
     specparam
       tplhl$A0$Y = 0.054:0.054:0.054,
       tphlh$A0$Y = 0.043:0.043:0.043,
       tplhl$A1$Y = 0.058:0.058:0.058,
       tphlh$A1$Y = 0.05:0.05:0.05,
       tplhl$B0$Y = 0.025:0.026:0.027,
       tphlh$B0$Y = 0.025:0.03:0.036;

     // path delays
     (A0 *> Y) = (tphlh$A0$Y, tplhl$A0$Y);
     (A1 *> Y) = (tphlh$A1$Y, tplhl$A1$Y);
     (B0 *> Y) = (tphlh$B0$Y, tplhl$B0$Y);

   endspecify

endmodule
`endcelldefine

`timescale 1ns/10ps
`celldefine
module AOI22X1 (A0, A1, B0, B1, Y);
input  A0 ;
input  A1 ;
input  B0 ;
input  B1 ;
output Y ;

   and (I0_out, A0, A1);
   and (I1_out, B0, B1);
   or  (I2_out, I0_out, I1_out);
   not (Y, I2_out);

   specify
     // delay parameters
     specparam
       tplhl$B1$Y = 0.077:0.083:0.09,
       tphlh$B1$Y = 0.048:0.056:0.063,
       tplhl$B0$Y = 0.073:0.08:0.087,
       tphlh$B0$Y = 0.043:0.05:0.057,
       tplhl$A1$Y = 0.045:0.047:0.049,
       tphlh$A1$Y = 0.031:0.037:0.043,
       tplhl$A0$Y = 0.043:0.044:0.046,
       tphlh$A0$Y = 0.027:0.032:0.038;

     // path delays
     (A0 *> Y) = (tphlh$A0$Y, tplhl$A0$Y);
     (A1 *> Y) = (tphlh$A1$Y, tplhl$A1$Y);
     (B0 *> Y) = (tphlh$B0$Y, tplhl$B0$Y);
     (B1 *> Y) = (tphlh$B1$Y, tplhl$B1$Y);

   endspecify

endmodule
`endcelldefine

`timescale 1ns/10ps
`celldefine
module BUFX1 (A, Y);
input  A ;
output Y ;

   buf (Y, A);

   specify
     // delay parameters
     specparam
       tpllh$A$Y = 0.048:0.048:0.048,
       tphhl$A$Y = 0.052:0.052:0.052;

     // path delays
     (A *> Y) = (tpllh$A$Y, tphhl$A$Y);

   endspecify

endmodule
`endcelldefine

`timescale 1ns/10ps
`celldefine
module BUFX3 (A, Y);
input  A ;
output Y ;

   buf (Y, A);

   specify
     // delay parameters
     specparam
       tpllh$A$Y = 0.057:0.057:0.057,
       tphhl$A$Y = 0.06:0.06:0.06;

     // path delays
     (A *> Y) = (tpllh$A$Y, tphhl$A$Y);

   endspecify

endmodule
`endcelldefine

`timescale 1ns/10ps
`celldefine
module CLKBUFX1 (A, Y);
input  A ;
output Y ;

   buf (Y, A);

   specify
     // delay parameters
     specparam
       tpllh$A$Y = 0.044:0.044:0.044,
       tphhl$A$Y = 0.047:0.047:0.047;

     // path delays
     (A *> Y) = (tpllh$A$Y, tphhl$A$Y);

   endspecify

endmodule
`endcelldefine

`timescale 1ns/10ps
`celldefine
module CLKBUFX2 (A, Y);
input  A ;
output Y ;

   buf (Y, A);

   specify
     // delay parameters
     specparam
       tpllh$A$Y = 0.042:0.042:0.042,
       tphhl$A$Y = 0.045:0.045:0.045;

     // path delays
     (A *> Y) = (tpllh$A$Y, tphhl$A$Y);

   endspecify

endmodule
`endcelldefine

`timescale 1ns/10ps
`celldefine
module CLKBUFX3 (A, Y);
input  A ;
output Y ;

   buf (Y, A);

   specify
     // delay parameters
     specparam
       tpllh$A$Y = 0.044:0.044:0.044,
       tphhl$A$Y = 0.047:0.047:0.047;

     // path delays
     (A *> Y) = (tpllh$A$Y, tphhl$A$Y);

   endspecify

endmodule
`endcelldefine

`timescale 1ns/10ps
`celldefine
module DFFSRX1 (CK, D, RN, SN, Q, QN);
input  CK ;
input  D ;
input  RN ;
input  SN ;
output Q ;
output QN ;
reg NOTIFIER ;

	not _i0 (_n1,RN);
	not _i1 (_n2,SN);
	udp_dff _i2 (Q, D, CK, _n1, _n2, NOTIFIER);
	not _i3 (QN,Q);
   specify
     // delay parameters
     specparam
       tpllh$SN$QN = 0.079:0.079:0.079,
       tphhl$SN$QN = 0.065:0.066:0.066,
       tphlh$SN$Q = 0.18:0.21:0.23,
       tpllh$CK$QN = 0.17:0.17:0.17,
       tplhl$CK$QN = 0.13:0.13:0.13,
       tpllh$CK$Q = 0.091:0.091:0.091,
       tplhl$CK$Q = 0.098:0.098:0.098,
       tpllh$RN$Q = 0.13:0.14:0.15,
       tphhl$RN$Q = 0.1:0.11:0.12,
       tphlh$RN$QN = 0.17:0.18:0.18,
       tminpwl$SN = 0.028:0.13:0.23,
       tminpwh$CK = 0.091:0.13:0.17,
       tminpwl$CK = 0.08:0.1:0.12,
       tminpwl$RN = 0.028:0.11:0.18,
       tsetup_negedge$D$CK = 0.062:0.062:0.062,
       thold_negedge$D$CK = 0.063:0.063:0.063,
       tsetup_posedge$D$CK = 0.12:0.12:0.12,
       thold_posedge$D$CK = 0:0:0,
       trec$RN$CK = 0.12:0.12:0.12,
       trem$RN$CK = 0:0:0,
       trec$RN$SN = 0:0.031:0.063,
       trec$SN$CK = 0:0:0,
       trem$SN$CK = 0.063:0.063:0.063,
       trec$SN$RN = 0.063:0.063:0.063;

     // path delays
     (CK *> Q) = (tpllh$CK$Q, tplhl$CK$Q);
     (CK *> QN) = (tpllh$CK$QN, tplhl$CK$QN);
     (RN *> Q) = (tpllh$RN$Q, tphhl$RN$Q);
     (RN *> QN) = (tphlh$RN$QN, 0);
     (SN *> Q) = (tphlh$SN$Q, 0);
     (SN *> QN) = (tpllh$SN$QN, tphhl$SN$QN);
     $setup(negedge D, posedge CK &&& \SN&RN , tsetup_negedge$D$CK, NOTIFIER);
     $hold (negedge D, posedge CK &&& \SN&RN , thold_negedge$D$CK,  NOTIFIER);
     $setup(posedge D, posedge CK &&& \SN&RN , tsetup_posedge$D$CK, NOTIFIER);
     $hold (posedge D, posedge CK &&& \SN&RN , thold_posedge$D$CK,  NOTIFIER);
     $recovery(posedge RN, posedge CK &&& \D&SN , trec$RN$CK, NOTIFIER);
     $removal (posedge RN, posedge CK &&& \D&SN , trem$RN$CK, NOTIFIER);
     $recovery(posedge RN, posedge SN, trec$RN$SN, NOTIFIER);
     $recovery(posedge SN, posedge CK &&& \~D&RN , trec$SN$CK, NOTIFIER);
     $removal (posedge SN, posedge CK &&& \~D&RN , trem$SN$CK, NOTIFIER);
     $recovery(posedge SN, posedge RN, trec$SN$RN, NOTIFIER);
     $width(negedge SN, tminpwl$SN, 0, NOTIFIER);
     $width(posedge CK, tminpwh$CK, 0, NOTIFIER);
     $width(negedge CK, tminpwl$CK, 0, NOTIFIER);
     $width(negedge RN, tminpwl$RN, 0, NOTIFIER);

   endspecify

endmodule
`endcelldefine

`timescale 1ns/10ps
`celldefine
module DFFX1 (CK, D, Q, QN);
input  CK ;
input  D ;
output Q ;
output QN ;
reg NOTIFIER ;

   udp_dff (P0002, D, CK, 1'B0, 1'B0, NOTIFIER);
   not (P0001, P0002);
   not (Q, P0001);
   buf (QN, P0001);

   specify
     // delay parameters
     specparam
       tpllh$CK$QN = 0.17:0.17:0.17,
       tplhl$CK$QN = 0.15:0.15:0.15,
       tpllh$CK$Q = 0.13:0.13:0.13,
       tplhl$CK$Q = 0.15:0.15:0.15,
       tminpwh$CK = 0.062:0.12:0.17,
       tminpwl$CK = 0.072:0.094:0.12,
       tsetup_negedge$D$CK = 0.12:0.12:0.12,
       thold_negedge$D$CK = 0:0:0,
       tsetup_posedge$D$CK = 0.12:0.12:0.12,
       thold_posedge$D$CK = 0:0:0;

     // path delays
     (CK *> Q) = (tpllh$CK$Q, tplhl$CK$Q);
     (CK *> QN) = (tpllh$CK$QN, tplhl$CK$QN);
     $setup(negedge D, posedge CK, tsetup_negedge$D$CK, NOTIFIER);
     $hold (negedge D, posedge CK, thold_negedge$D$CK,  NOTIFIER);
     $setup(posedge D, posedge CK, tsetup_posedge$D$CK, NOTIFIER);
     $hold (posedge D, posedge CK, thold_posedge$D$CK,  NOTIFIER);
     $width(posedge CK, tminpwh$CK, 0, NOTIFIER);
     $width(negedge CK, tminpwl$CK, 0, NOTIFIER);

   endspecify

endmodule
`endcelldefine

`timescale 1ns/10ps
`celldefine
module INVX1 (A, Y);
input  A ;
output Y ;

   not (Y, A);

   specify
     // delay parameters
     specparam
       tplhl$A$Y = 0.022:0.022:0.022,
       tphlh$A$Y = 0.022:0.022:0.022;

     // path delays
     (A *> Y) = (tphlh$A$Y, tplhl$A$Y);

   endspecify

endmodule
`endcelldefine

`timescale 1ns/10ps
`celldefine
module INVX2 (A, Y);
input  A ;
output Y ;

   not (Y, A);

   specify
     // delay parameters
     specparam
       tplhl$A$Y = 0.019:0.019:0.019,
       tphlh$A$Y = 0.018:0.018:0.018;

     // path delays
     (A *> Y) = (tphlh$A$Y, tplhl$A$Y);

   endspecify

endmodule
`endcelldefine

`timescale 1ns/10ps
`celldefine
module INVX4 (A, Y);
input  A ;
output Y ;

   not (Y, A);

   specify
     // delay parameters
     specparam
       tplhl$A$Y = 0.016:0.016:0.016,
       tphlh$A$Y = 0.016:0.016:0.016;

     // path delays
     (A *> Y) = (tphlh$A$Y, tplhl$A$Y);

   endspecify

endmodule
`endcelldefine

`timescale 1ns/10ps
`celldefine
module INVX8 (A, Y);
input  A ;
output Y ;

   not (Y, A);

   specify
     // delay parameters
     specparam
       tplhl$A$Y = 0.015:0.015:0.015,
       tphlh$A$Y = 0.015:0.015:0.015;

     // path delays
     (A *> Y) = (tphlh$A$Y, tplhl$A$Y);

   endspecify

endmodule
`endcelldefine

`timescale 1ns/10ps
`celldefine
module MX2X1 (A, B, S0, Y);
input  A ;
input  B ;
input  S0 ;
output Y ;

   udp_mux2 (Y, A, B, S0);

   specify
     // delay parameters
     specparam
       tpllh$S0$Y = 0.076:0.076:0.076,
       tplhl$S0$Y = 0.076:0.076:0.076,
       tpllh$A$Y = 0.082:0.082:0.082,
       tphhl$A$Y = 0.08:0.08:0.08,
       tpllh$B$Y = 0.082:0.082:0.082,
       tphhl$B$Y = 0.082:0.082:0.082;

     // path delays
     (A *> Y) = (tpllh$A$Y, tphhl$A$Y);
     (B *> Y) = (tpllh$B$Y, tphhl$B$Y);
     (S0 *> Y) = (tpllh$S0$Y, tplhl$S0$Y);

   endspecify

endmodule
`endcelldefine

`timescale 1ns/10ps
`celldefine
module NAND2X1 (A, B, Y);
input  A ;
input  B ;
output Y ;

   and (I0_out, A, B);
   not (Y, I0_out);

   specify
     // delay parameters
     specparam
       tplhl$B$Y = 0.036:0.036:0.036,
       tphlh$B$Y = 0.03:0.03:0.03,
       tplhl$A$Y = 0.034:0.034:0.034,
       tphlh$A$Y = 0.025:0.025:0.025;

     // path delays
     (A *> Y) = (tphlh$A$Y, tplhl$A$Y);
     (B *> Y) = (tphlh$B$Y, tplhl$B$Y);

   endspecify

endmodule
`endcelldefine

`timescale 1ns/10ps
`celldefine
module NAND2X2 (A, B, Y);
input  A ;
input  B ;
output Y ;

   and (I0_out, A, B);
   not (Y, I0_out);

   specify
     // delay parameters
     specparam
       tplhl$B$Y = 0.032:0.032:0.032,
       tphlh$B$Y = 0.025:0.025:0.025,
       tplhl$A$Y = 0.029:0.029:0.029,
       tphlh$A$Y = 0.021:0.021:0.021;

     // path delays
     (A *> Y) = (tphlh$A$Y, tplhl$A$Y);
     (B *> Y) = (tphlh$B$Y, tplhl$B$Y);

   endspecify

endmodule
`endcelldefine

`timescale 1ns/10ps
`celldefine
module NAND3X1 (A, B, C, Y);
input  A ;
input  B ;
input  C ;
output Y ;

   and (I1_out, A, B, C);
   not (Y, I1_out);

   specify
     // delay parameters
     specparam
       tplhl$C$Y = 0.038:0.038:0.038,
       tphlh$C$Y = 0.048:0.048:0.048,
       tplhl$A$Y = 0.028:0.028:0.028,
       tphlh$A$Y = 0.032:0.032:0.032,
       tplhl$B$Y = 0.034:0.034:0.034,
       tphlh$B$Y = 0.041:0.041:0.041;

     // path delays
     (A *> Y) = (tphlh$A$Y, tplhl$A$Y);
     (B *> Y) = (tphlh$B$Y, tplhl$B$Y);
     (C *> Y) = (tphlh$C$Y, tplhl$C$Y);

   endspecify

endmodule
`endcelldefine

`timescale 1ns/10ps
`celldefine
module NAND4X1 (A, B, C, D, Y);
input  A ;
input  B ;
input  C ;
input  D ;
output Y ;

   and (I2_out, A, B, C, D);
   not (Y, I2_out);

   specify
     // delay parameters
     specparam
       tplhl$B$Y = 0.039:0.039:0.039,
       tphlh$B$Y = 0.047:0.047:0.047,
       tplhl$D$Y = 0.05:0.05:0.05,
       tphlh$D$Y = 0.064:0.064:0.064,
       tplhl$A$Y = 0.03:0.03:0.03,
       tphlh$A$Y = 0.037:0.037:0.037,
       tplhl$C$Y = 0.046:0.046:0.046,
       tphlh$C$Y = 0.057:0.057:0.057;

     // path delays
     (A *> Y) = (tphlh$A$Y, tplhl$A$Y);
     (B *> Y) = (tphlh$B$Y, tplhl$B$Y);
     (C *> Y) = (tphlh$C$Y, tplhl$C$Y);
     (D *> Y) = (tphlh$D$Y, tplhl$D$Y);

   endspecify

endmodule
`endcelldefine

`timescale 1ns/10ps
`celldefine
module NOR2X1 (A, B, Y);
input  A ;
input  B ;
output Y ;

   or  (I0_out, A, B);
   not (Y, I0_out);

   specify
     // delay parameters
     specparam
       tplhl$B$Y = 0.027:0.027:0.027,
       tphlh$B$Y = 0.029:0.029:0.029,
       tplhl$A$Y = 0.037:0.037:0.037,
       tphlh$A$Y = 0.039:0.039:0.039;

     // path delays
     (A *> Y) = (tphlh$A$Y, tplhl$A$Y);
     (B *> Y) = (tphlh$B$Y, tplhl$B$Y);

   endspecify

endmodule
`endcelldefine

`timescale 1ns/10ps
`celldefine
module NOR3X1 (A, B, C, Y);
input  A ;
input  B ;
input  C ;
output Y ;

   or  (I1_out, A, B, C);
   not (Y, I1_out);

   specify
     // delay parameters
     specparam
       tplhl$B$Y = 0.067:0.067:0.067,
       tphlh$B$Y = 0.047:0.047:0.047,
       tplhl$C$Y = 0.039:0.039:0.039,
       tphlh$C$Y = 0.026:0.026:0.026,
       tplhl$A$Y = 0.084:0.084:0.084,
       tphlh$A$Y = 0.059:0.059:0.059;

     // path delays
     (A *> Y) = (tphlh$A$Y, tplhl$A$Y);
     (B *> Y) = (tphlh$B$Y, tplhl$B$Y);
     (C *> Y) = (tphlh$C$Y, tplhl$C$Y);

   endspecify

endmodule
`endcelldefine

`timescale 1ns/10ps
`celldefine
module NOR4X1 (A, B, C, D, Y);
input  A ;
input  B ;
input  C ;
input  D ;
output Y ;

   or  (I2_out, A, B, C, D);
   not (Y, I2_out);

   specify
     // delay parameters
     specparam
       tplhl$C$Y = 0.12:0.12:0.12,
       tphlh$C$Y = 0.13:0.13:0.13,
       tplhl$B$Y = 0.14:0.14:0.14,
       tphlh$B$Y = 0.15:0.15:0.15,
       tplhl$D$Y = 0.092:0.092:0.092,
       tphlh$D$Y = 0.094:0.094:0.094,
       tplhl$A$Y = 0.15:0.15:0.15,
       tphlh$A$Y = 0.17:0.17:0.17;

     // path delays
     (A *> Y) = (tphlh$A$Y, tplhl$A$Y);
     (B *> Y) = (tphlh$B$Y, tplhl$B$Y);
     (C *> Y) = (tphlh$C$Y, tplhl$C$Y);
     (D *> Y) = (tphlh$D$Y, tplhl$D$Y);

   endspecify

endmodule
`endcelldefine

`timescale 1ns/10ps
`celldefine
module OAI21X1 (A0, A1, B0, Y);
input  A0 ;
input  A1 ;
input  B0 ;
output Y ;

   or  (I0_out, A0, A1);
   and (I1_out, I0_out, B0);
   not (Y, I1_out);

   specify
     // delay parameters
     specparam
       tplhl$B0$Y = 0.033:0.047:0.06,
       tphlh$B0$Y = 0.021:0.022:0.022,
       tplhl$A1$Y = 0.065:0.065:0.065,
       tphlh$A1$Y = 0.049:0.049:0.049,
       tplhl$A0$Y = 0.046:0.046:0.046,
       tphlh$A0$Y = 0.039:0.039:0.039;

     // path delays
     (A0 *> Y) = (tphlh$A0$Y, tplhl$A0$Y);
     (A1 *> Y) = (tphlh$A1$Y, tplhl$A1$Y);
     (B0 *> Y) = (tphlh$B0$Y, tplhl$B0$Y);

   endspecify

endmodule
`endcelldefine

`timescale 1ns/10ps
`celldefine
module OAI22X1 (A0, A1, B0, B1, Y);
input  A0 ;
input  A1 ;
input  B0 ;
input  B1 ;
output Y ;

   or  (I0_out, B0, B1);
   or  (I1_out, A0, A1);
   and (I2_out, I0_out, I1_out);
   not (Y, I2_out);

   specify
     // delay parameters
     specparam
       tplhl$A1$Y = 0.097:0.11:0.13,
       tphlh$A1$Y = 0.11:0.12:0.12,
       tplhl$B1$Y = 0.11:0.12:0.14,
       tphlh$B1$Y = 0.13:0.14:0.14,
       tplhl$A0$Y = 0.086:0.099:0.11,
       tphlh$A0$Y = 0.1:0.11:0.11,
       tplhl$B0$Y = 0.094:0.11:0.12,
       tphlh$B0$Y = 0.12:0.13:0.13;

     // path delays
     (A0 *> Y) = (tphlh$A0$Y, tplhl$A0$Y);
     (A1 *> Y) = (tphlh$A1$Y, tplhl$A1$Y);
     (B0 *> Y) = (tphlh$B0$Y, tplhl$B0$Y);
     (B1 *> Y) = (tphlh$B1$Y, tplhl$B1$Y);

   endspecify

endmodule
`endcelldefine

`timescale 1ns/10ps
`celldefine
module OAI33X1 (A0, A1, A2, B0, B1, B2, Y);
input  A0 ;
input  A1 ;
input  A2 ;
input  B0 ;
input  B1 ;
input  B2 ;
output Y ;

   or  (I1_out, B0, B1, B2);
   or  (I3_out, A0, A1, A2);
   and (I4_out, I1_out, I3_out);
   not (Y, I4_out);

   specify
     // delay parameters
     specparam
       tplhl$A2$Y = 0.12:0.15:0.19,
       tphlh$A2$Y = 0.15:0.16:0.17,
       tplhl$B0$Y = 0.1:0.13:0.16,
       tphlh$B0$Y = 0.14:0.15:0.16,
       tplhl$B1$Y = 0.12:0.15:0.19,
       tphlh$B1$Y = 0.16:0.17:0.18,
       tplhl$A0$Y = 0.091:0.12:0.15,
       tphlh$A0$Y = 0.11:0.12:0.13,
       tplhl$A1$Y = 0.11:0.14:0.17,
       tphlh$A1$Y = 0.14:0.14:0.15,
       tplhl$B2$Y = 0.13:0.17:0.2,
       tphlh$B2$Y = 0.17:0.18:0.19;

     // path delays
     (A0 *> Y) = (tphlh$A0$Y, tplhl$A0$Y);
     (A1 *> Y) = (tphlh$A1$Y, tplhl$A1$Y);
     (A2 *> Y) = (tphlh$A2$Y, tplhl$A2$Y);
     (B0 *> Y) = (tphlh$B0$Y, tplhl$B0$Y);
     (B1 *> Y) = (tphlh$B1$Y, tplhl$B1$Y);
     (B2 *> Y) = (tphlh$B2$Y, tplhl$B2$Y);

   endspecify

endmodule
`endcelldefine

`timescale 1ns/10ps
`celldefine
module OR2X1 (A, B, Y);
input  A ;
input  B ;
output Y ;

   or  (Y, A, B);

   specify
     // delay parameters
     specparam
       tpllh$A$Y = 0.056:0.056:0.056,
       tphhl$A$Y = 0.075:0.075:0.075,
       tpllh$B$Y = 0.047:0.047:0.047,
       tphhl$B$Y = 0.066:0.066:0.066;

     // path delays
     (A *> Y) = (tpllh$A$Y, tphhl$A$Y);
     (B *> Y) = (tpllh$B$Y, tphhl$B$Y);

   endspecify

endmodule
`endcelldefine

`timescale 1ns/10ps
`celldefine
module OR4X1 (A, B, C, D, Y);
input  A ;
input  B ;
input  C ;
input  D ;
output Y ;

   or  (Y, A, B, C, D);

   specify
     // delay parameters
     specparam
       tpllh$D$Y = 0.063:0.063:0.063,
       tphhl$D$Y = 0.066:0.066:0.066,
       tpllh$A$Y = 0.11:0.11:0.11,
       tphhl$A$Y = 0.13:0.13:0.13,
       tpllh$C$Y = 0.089:0.089:0.089,
       tphhl$C$Y = 0.1:0.1:0.1,
       tpllh$B$Y = 0.11:0.11:0.11,
       tphhl$B$Y = 0.12:0.12:0.12;

     // path delays
     (A *> Y) = (tpllh$A$Y, tphhl$A$Y);
     (B *> Y) = (tpllh$B$Y, tphhl$B$Y);
     (C *> Y) = (tpllh$C$Y, tphhl$C$Y);
     (D *> Y) = (tpllh$D$Y, tphhl$D$Y);

   endspecify

endmodule
`endcelldefine

`timescale 1ns/10ps
`celldefine
module SDFFSRX1 (CK, D, RN, SE, SI, SN, Q, QN);
input  CK ;
input  D ;
input  RN ;
input  SE ;
input  SI ;
input  SN ;
output Q ;
output QN ;
reg NOTIFIER ;

   udp_mux2 (I0_D, D, SI, SE);
   not (I0_CLEAR, RN);
   not (I0_SET, SN);
   udp_dff (P0002, I0_D, CK, I0_CLEAR, I0_SET, NOTIFIER);
   not (P0000, P0002);
   not (Q, P0000);
   buf (QN, P0000);
   not (I7_out, SE);
   and (\~SE&RN&SN , I7_out, RN, SN);
   and (\RN&SN , RN, SN);
   and (\SE&RN&SN , SE, RN, SN);

   specify
     // delay parameters
     specparam
       tphlh$RN$QN = 0.14:0.14:0.14,
       tphhl$RN$Q = 0.18:0.18:0.18,
       tpllh$SN$QN = 0.12:0.12:0.12,
       tphhl$SN$QN = 0.12:0.12:0.12,
       tplhl$SN$Q = 0.16:0.16:0.16,
       tphlh$SN$Q = 0.15:0.16:0.16,
       tpllh$CK$QN = 0.29:0.29:0.29,
       tplhl$CK$QN = 0.4:0.4:0.4,
       tpllh$CK$Q = 0.45:0.45:0.45,
       tplhl$CK$Q = 0.33:0.33:0.33,
       tminpwl$RN = 0.094:0.17:0.24,
       tminpwl$SN = 0.086:0.12:0.16,
       tminpwh$CK = 0.15:0.3:0.45,
       tminpwl$CK = 0.13:0.27:0.41,
       tsetup_negedge$D$CK = 0.31:0.31:0.31,
       thold_negedge$D$CK = -0.19:-0.19:-0.19,
       tsetup_negedge$SE$CK = 0.19:0.25:0.31,
       thold_negedge$SE$CK = -0.19:-0.12:-0.062,
       tsetup_negedge$SI$CK = 0.31:0.31:0.31,
       thold_negedge$SI$CK = -0.19:-0.19:-0.19,
       tsetup_posedge$D$CK = 0.12:0.12:0.12,
       thold_posedge$D$CK = 0:0:0,
       tsetup_posedge$SE$CK = 0.12:0.22:0.31,
       thold_posedge$SE$CK = -0.19:-0.12:-0.062,
       tsetup_posedge$SI$CK = 0.12:0.12:0.12,
       thold_posedge$SI$CK = -0.062:-0.062:-0.062,
       trec$RN$CK = 0.062:0.062:0.062,
       trem$RN$CK = 0.063:0.063:0.063,
       trec$RN$SN = 0.12:0.12:0.12,
       trec$SN$CK = 0.062:0.062:0.062,
       trem$SN$CK = 0.12:0.12:0.12;

     // path delays
     (CK *> Q) = (tpllh$CK$Q, tplhl$CK$Q);
     (CK *> QN) = (tpllh$CK$QN, tplhl$CK$QN);
     (RN *> Q) = (0, tphhl$RN$Q);
     (RN *> QN) = (tphlh$RN$QN, 0);
     (SN *> Q) = (tphlh$SN$Q, tplhl$SN$Q);
     (SN *> QN) = (tpllh$SN$QN, tphhl$SN$QN);
     $setup(negedge D, posedge CK &&& \~SE&RN&SN , tsetup_negedge$D$CK, NOTIFIER);
     $hold (negedge D, posedge CK &&& \~SE&RN&SN , thold_negedge$D$CK,  NOTIFIER);
     $setup(negedge SE, posedge CK &&& \RN&SN , tsetup_negedge$SE$CK, NOTIFIER);
     $hold (negedge SE, posedge CK &&& \RN&SN , thold_negedge$SE$CK,  NOTIFIER);
     $setup(negedge SI, posedge CK &&& \SE&RN&SN , tsetup_negedge$SI$CK, NOTIFIER);
     $hold (negedge SI, posedge CK &&& \SE&RN&SN , thold_negedge$SI$CK,  NOTIFIER);
     $setup(posedge D, posedge CK &&& \~SE&RN&SN , tsetup_posedge$D$CK, NOTIFIER);
     $hold (posedge D, posedge CK &&& \~SE&RN&SN , thold_posedge$D$CK,  NOTIFIER);
     $setup(posedge SE, posedge CK &&& \RN&SN , tsetup_posedge$SE$CK, NOTIFIER);
     $hold (posedge SE, posedge CK &&& \RN&SN , thold_posedge$SE$CK,  NOTIFIER);
     $setup(posedge SI, posedge CK &&& \SE&RN&SN , tsetup_posedge$SI$CK, NOTIFIER);
     $hold (posedge SI, posedge CK &&& \SE&RN&SN , thold_posedge$SI$CK,  NOTIFIER);
     $recovery(posedge RN, posedge CK &&& SN, trec$RN$CK, NOTIFIER);
     $removal (posedge RN, posedge CK &&& SN, trem$RN$CK, NOTIFIER);
     $recovery(posedge RN, posedge SN, trec$RN$SN, NOTIFIER);
     $recovery(posedge SN, posedge CK &&& RN, trec$SN$CK, NOTIFIER);
     $removal (posedge SN, posedge CK &&& RN, trem$SN$CK, NOTIFIER);
     $width(negedge RN, tminpwl$RN, 0, NOTIFIER);
     $width(negedge SN, tminpwl$SN, 0, NOTIFIER);
     $width(posedge CK, tminpwh$CK, 0, NOTIFIER);
     $width(negedge CK, tminpwl$CK, 0, NOTIFIER);

   endspecify

endmodule
`endcelldefine

`timescale 1ns/10ps
`celldefine
module TBUFX1 (A, OE, Y);
input  A ;
input  OE ;
output Y ;

   bufif1 (Y, A, OE);

   specify
     // delay parameters
     specparam
       tpzh$OE$Y = 0.084:0.084:0.084,
       tpzl$OE$Y = 0.04:0.04:0.04,
       tplz$OE$Y = 0.045:0.045:0.045,
       tphz$OE$Y = 0.048:0.048:0.048,
       tpllh$A$Y = 0.066:0.066:0.066,
       tphhl$A$Y = 0.062:0.062:0.062;

     // path delays
     (A *> Y) = (tpllh$A$Y, tphhl$A$Y);
     (OE *> Y) = (0, 0, tplz$OE$Y, tpzh$OE$Y, tphz$OE$Y, tpzl$OE$Y);

   endspecify

endmodule
`endcelldefine

`timescale 1ns/10ps
`celldefine
module TBUFX2 (A, OE, Y);
input  A ;
input  OE ;
output Y ;

   bufif1 (Y, A, OE);

   specify
     // delay parameters
     specparam
       tpllh$A$Y = 0.082:0.082:0.082,
       tphhl$A$Y = 0.073:0.073:0.073,
       tpzh$OE$Y = 0.099:0.099:0.099,
       tpzl$OE$Y = 0.034:0.034:0.034,
       tplz$OE$Y = 0.045:0.045:0.045,
       tphz$OE$Y = 0.071:0.071:0.071;

     // path delays
     (A *> Y) = (tpllh$A$Y, tphhl$A$Y);
     (OE *> Y) = (0, 0, tplz$OE$Y, tpzh$OE$Y, tphz$OE$Y, tpzl$OE$Y);

   endspecify

endmodule
`endcelldefine

`timescale 1ns/10ps
`celldefine
module TBUFX4 (A, OE, Y);
input  A ;
input  OE ;
output Y ;

   bufif1 (Y, A, OE);

   specify
     // delay parameters
     specparam
       tpllh$A$Y = 0.079:0.079:0.079,
       tphhl$A$Y = 0.07:0.07:0.07,
       tpzh$OE$Y = 0.096:0.096:0.096,
       tpzl$OE$Y = 0.036:0.036:0.036,
       tplz$OE$Y = 0.045:0.045:0.045,
       tphz$OE$Y = 0.066:0.066:0.066;

     // path delays
     (A *> Y) = (tpllh$A$Y, tphhl$A$Y);
     (OE *> Y) = (0, 0, tplz$OE$Y, tpzh$OE$Y, tphz$OE$Y, tpzl$OE$Y);

   endspecify

endmodule
`endcelldefine

`timescale 1ns/10ps
`celldefine
module TBUFX8 (A, OE, Y);
input  A ;
input  OE ;
output Y ;

   bufif1 (Y, A, OE);

   specify
     // delay parameters
     specparam
       tpllh$A$Y = 0.076:0.076:0.076,
       tphhl$A$Y = 0.066:0.066:0.066,
       tpzh$OE$Y = 0.094:0.094:0.094,
       tpzl$OE$Y = 0.031:0.031:0.031,
       tplz$OE$Y = 0.045:0.045:0.045,
       tphz$OE$Y = 0.069:0.069:0.069;

     // path delays
     (A *> Y) = (tpllh$A$Y, tphhl$A$Y);
     (OE *> Y) = (0, 0, tplz$OE$Y, tpzh$OE$Y, tphz$OE$Y, tpzl$OE$Y);

   endspecify

endmodule
`endcelldefine

`timescale 1ns/10ps
`celldefine
module TINVX1 (A, OE, Y);
input  A ;
input  OE ;
output Y ;

   not (I0_out, A);
   bufif1 (Y, I0_out, OE);

   specify
     // delay parameters
     specparam
       tpzh$OE$Y = 0.07:0.07:0.07,
       tpzl$OE$Y = 0.029:0.029:0.029,
       tplz$OE$Y = 0.045:0.045:0.045,
       tphz$OE$Y = 0.063:0.063:0.063,
       tplhl$A$Y = 0.036:0.036:0.036,
       tphlh$A$Y = 0.049:0.049:0.049;

     // path delays
     (A *> Y) = (tphlh$A$Y, tplhl$A$Y);
     (OE *> Y) = (0, 0, tplz$OE$Y, tpzh$OE$Y, tphz$OE$Y, tpzl$OE$Y);

   endspecify

endmodule
`endcelldefine

`timescale 1ns/10ps
`celldefine
module TLATSRX1 (D, G, RN, SN, Q, QN);
input  D ;
input  G ;
input  RN ;
input  SN ;
output Q ;
output QN ;
reg NOTIFIER ;

   not (I0_CLEAR, RN);
   not (I0_SET, SN);
   udp_tlat (P0001, D, G, I0_CLEAR, I0_SET, NOTIFIER);
   not (P0000, P0001);
   not (Q, P0000);
   buf (QN, P0000);
   and (\D&SN , D, SN);
   not (I7_out, D);
   and (\~D&RN , I7_out, RN);
   and (\SN&RN , SN, RN);

   specify
     // delay parameters
     specparam
       tpllh$D$Q = 0.15:0.15:0.15,
       tphhl$D$Q = 0.19:0.19:0.19,
       tplhl$D$QN = 0.23:0.23:0.23,
       tphlh$D$QN = 0.24:0.24:0.24,
       tpllh$G$Q = 0.23:0.23:0.23,
       tplhl$G$Q = 0.18:0.18:0.18,
       tpllh$G$QN = 0.23:0.23:0.23,
       tplhl$G$QN = 0.31:0.31:0.31,
       tplhl$SN$Q = 0.18:0.18:0.18,
       tphlh$SN$Q = 0.22:0.22:0.22,
       tpllh$SN$QN = 0.24:0.24:0.24,
       tphhl$SN$QN = 0.29:0.29:0.29,
       tpllh$RN$Q = 0.15:0.15:0.15,
       tphhl$RN$Q = 0.14:0.14:0.14,
       tplhl$RN$QN = 0.23:0.23:0.23,
       tphlh$RN$QN = 0.2:0.2:0.2,
       tminpwh$G = 0.14:0.22:0.31,
       tminpwl$SN = 0.18:0.23:0.29,
       tminpwl$RN = 0.1:0.15:0.2,
       tsetup_negedge$D$G = 0.19:0.19:0.19,
       thold_negedge$D$G = -0.12:-0.12:-0.12,
       tsetup_posedge$D$G = 0.12:0.12:0.12,
       thold_posedge$D$G = 0:0:0,
       trec$RN$G = 0.12:0.12:0.12,
       trem$RN$G = 0:0:0,
       trec$RN$SN = 0.25:0.25:0.25,
       trec$SN$G = 0.19:0.19:0.19,
       trem$SN$G = -0.12:-0.12:-0.12;

     // path delays
     (D *> Q) = (tpllh$D$Q, tphhl$D$Q);
     (D *> QN) = (tphlh$D$QN, tplhl$D$QN);
     (G *> Q) = (tpllh$G$Q, tplhl$G$Q);
     (G *> QN) = (tpllh$G$QN, tplhl$G$QN);
     (RN *> Q) = (tpllh$RN$Q, tphhl$RN$Q);
     (RN *> QN) = (tphlh$RN$QN, tplhl$RN$QN);
     (SN *> Q) = (tphlh$SN$Q, tplhl$SN$Q);
     (SN *> QN) = (tpllh$SN$QN, tphhl$SN$QN);
     $setup(negedge D, negedge G &&& \SN&RN , tsetup_negedge$D$G, NOTIFIER);
     $hold (negedge D, negedge G &&& \SN&RN , thold_negedge$D$G,  NOTIFIER);
     $setup(posedge D, negedge G &&& \SN&RN , tsetup_posedge$D$G, NOTIFIER);
     $hold (posedge D, negedge G &&& \SN&RN , thold_posedge$D$G,  NOTIFIER);
     $recovery(posedge RN, negedge G &&& \D&SN , trec$RN$G, NOTIFIER);
     $removal (posedge RN, negedge G &&& \D&SN , trem$RN$G, NOTIFIER);
     $recovery(posedge RN, posedge SN, trec$RN$SN, NOTIFIER);
     $recovery(posedge SN, negedge G &&& \~D&RN , trec$SN$G, NOTIFIER);
     $removal (posedge SN, negedge G &&& \~D&RN , trem$SN$G, NOTIFIER);
     $width(posedge G, tminpwh$G, 0, NOTIFIER);
     $width(negedge SN, tminpwl$SN, 0, NOTIFIER);
     $width(negedge RN, tminpwl$RN, 0, NOTIFIER);

   endspecify

endmodule
`endcelldefine

`timescale 1ns/10ps
`celldefine
module TLATX1 (C, D, Q, QN);
input  C ;
input  D ;
output Q ;
output QN ;
reg NOTIFIER ;

   udp_tlat (P0001, D, C, 1'B0, 1'B0, NOTIFIER);
   not (P0000, P0001);
   buf (Q, P0001);
   not (QN, P0001);

   specify
     // delay parameters
     specparam
       tpllh$C$QN = 0.18:0.18:0.18,
       tplhl$C$QN = 0.19:0.19:0.19,
       tpllh$C$Q = 0.15:0.15:0.15,
       tplhl$C$Q = 0.14:0.14:0.14,
       tplhl$D$QN = 0.13:0.13:0.13,
       tphlh$D$QN = 0.17:0.17:0.17,
       tpllh$D$Q = 0.095:0.095:0.095,
       tphhl$D$Q = 0.13:0.13:0.13,
       tminpwh$C = 0.1:0.14:0.19,
       tsetup_negedge$D$C = 0.12:0.12:0.12,
       thold_negedge$D$C = -0.062:-0.062:-0.062,
       tsetup_posedge$D$C = 0.062:0.062:0.062,
       thold_posedge$D$C = 0:0:0;

     // path delays
     (C *> Q) = (tpllh$C$Q, tplhl$C$Q);
     (C *> QN) = (tpllh$C$QN, tplhl$C$QN);
     (D *> Q) = (tpllh$D$Q, tphhl$D$Q);
     (D *> QN) = (tphlh$D$QN, tplhl$D$QN);
     $setup(negedge D, negedge C, tsetup_negedge$D$C, NOTIFIER);
     $hold (negedge D, negedge C, thold_negedge$D$C,  NOTIFIER);
     $setup(posedge D, negedge C, tsetup_posedge$D$C, NOTIFIER);
     $hold (posedge D, negedge C, thold_posedge$D$C,  NOTIFIER);
     $width(posedge C, tminpwh$C, 0, NOTIFIER);

   endspecify

endmodule
`endcelldefine

`timescale 1ns/10ps
`celldefine
module XOR2X1 (A, B, Y);
input  A ;
input  B ;
output Y ;

   xor (Y, A, B);

   specify
     // delay parameters
     specparam
       tpllh$B$Y = 0.079:0.079:0.079,
       tplhl$B$Y = 0.1:0.1:0.1,
       tpllh$A$Y = 0.099:0.099:0.099,
       tplhl$A$Y = 0.11:0.11:0.11;

     // path delays
     (A *> Y) = (tpllh$A$Y, tplhl$A$Y);
     (B *> Y) = (tpllh$B$Y, tplhl$B$Y);

   endspecify

endmodule
`endcelldefine

`timescale 1ns/10ps
`celldefine
module ADDHX1 (A, B, CO, S);
input  A ;
input  B ;
output CO ;
output S ;

   and (CO, A, B);
   xor (S, A, B);

   specify
     // delay parameters
     specparam
       tpllh$A$S = 0.1:0.1:0.1,
       tplhl$A$S = 0.068:0.068:0.068,
       tpllh$A$CO = 0.062:0.062:0.062,
       tphhl$A$CO = 0.08:0.08:0.08,
       tpllh$B$S = 0.07:0.07:0.07,
       tplhl$B$S = 0.056:0.056:0.056,
       tpllh$B$CO = 0.058:0.058:0.058,
       tphhl$B$CO = 0.067:0.067:0.067;

     // path delays
     (A *> CO) = (tpllh$A$CO, tphhl$A$CO);
     (A *> S) = (tpllh$A$S, tplhl$A$S);
     (B *> CO) = (tpllh$B$CO, tphhl$B$CO);
     (B *> S) = (tpllh$B$S, tplhl$B$S);

   endspecify

endmodule
`endcelldefine

primitive udp_dff (out, in, clk, clr, set, NOTIFIER);
   output out;
   input  in, clk, clr, set, NOTIFIER;
   reg    out;

   table

// in  clk  clr   set  NOT  : Qt : Qt+1
//

         ?   ?    0    1    ?  : ? : 1 ; // set active
         ?   ?    1    ?    ?  : ? : 0 ;  // reset active (W/ OR W/O SET ACTIVE)
         1   r    0    0    ?  : ? : 1 ;  // normal clocking
         0   r    0    0    ?  : ? : 0 ;
         0  (BX)  ?    0    ?  : 0 : - ; // clock b->U,but D = Q
         1  (BX)  0    ?    ?  : 1 : - ;
         ?  (?0)  ?    ?    ?  : ? : - ; // ignore all clock->0 transitions
         1  (X1)  0    ?    ?  : 1 : - ; // clock going unknown to active D=Qo=1
         0  (X1)  ?    0    ?  : 0 : - ; // clock going unknown to active D=Qo=0
         0   r    x    0    ?  : ? : 0 ;  // reset unknown, but clocking in 0
         1   r    0    x    ?  : ? : 1 ;  // set unknown, but clocking in 1
         ?   ?   (?X)  1    ?  : 0 : x ; // reset going unknown
         ?   b   (?X)  0    ?  : 0 : - ;
         0   x   (?X)  0    ?  : 0 : - ;
         1   x   (?X)  b    ?  : 0 : x ;
         x   x   (?X)  b    ?  : 0 : x ;
         ?   ?    x    0    ?  : 1 : x ;
         ?   ?    ?   (?X)  ?  : 0 : x ; // set going unknown
         ?   b    b   (?X)  ?  : 1 : - ;
         0   x    b   (?X)  ?  : 1 : x ;
         1   x    b   (?X)  ?  : 1 : - ;
         x   x    b   (?X)  ?  : 1 : x ;
         ?   ?    0    x    ?  : 0 : x ;
         *   b    ?    ?    ?  : ? : - ;  // ignore data changes
         ?   b   (?0)  ?    ?  : ? : - ; // ignore reset falling
         0   ?   (?0)  ?    ?  : ? : - ;
         ?   b    ?   (?0)  ?  : ? : - ; // ignore set falling
         1   ?    ?   (?0)  ?  : ? : - ;
         ?   ?    ?    ?    *  : ? : x ;  // timing violation
   endtable
endprimitive // udp_dff

primitive udp_tlat (out, in, enable, clr, set, NOTIFIER);

   output out;
   input  in, enable, clr, set, NOTIFIER;
   reg    out;

   table

// in  enable  clr   set  NOT  : Qt : Qt+1
//
   1  1   0   ?   ?   : ?  :  1  ; //
   0  1   ?   0   ?   : ?  :  0  ; //
   1  *   0   ?   ?   : 1  :  1  ; // reduce pessimism
   0  *   ?   0   ?   : 0  :  0  ; // reduce pessimism
   *  0   ?   ?   ?   : ?  :  -  ; // no changes when in switches
   ?  ?   ?   1   ?   : ?  :  1  ; // set output
   ?  0   0   *   ?   : 1  :  1  ; // cover all transistions on set
   1  ?   0   *   ?   : 1  :  1  ; // cover all transistions on set
   ?  ?   1   0   ?   : ?  :  0  ; // reset output
   ?  0   *   0   ?   : 0  :  0  ; // cover all transistions on clr
   0  ?   *   0   ?   : 0  :  0  ; // cover all transistions on clr
   ?  ?   ?   ?   *   : ?  :  x  ; // any notifier changed

   endtable
endprimitive // udp_tlat

primitive udp_rslat (out, clr, set, NOTIFIER);

   output out;
   input  clr, set, NOTIFIER;
   reg    out;

   table

// clr   set  NOT  : Qt : Qt+1
//
   ?   1   ?   : ?  :  1  ; // set output
   0   *   ?   : 1  :  1  ; // cover all transistions on set
   1   0   ?   : ?  :  0  ; // reset output
   *   0   ?   : 0  :  0  ; // cover all transistions on clr
   ?   ?   *   : ?  :  x  ; // any notifier changed

   endtable
endprimitive // udp_tlat

primitive udp_mux2 (out, in0, in1, sel);
   output out;
   input  in0, in1, sel;

   table

// in0 in1 sel :  out
//
    1  ?  0 :  1 ;
    0  ?  0 :  0 ;
    ?  1  1 :  1 ;
    ?  0  1 :  0 ;
    0  0  x :  0 ;
    1  1  x :  1 ;

   endtable
endprimitive // udp_mux2

