// Created by ihdl
module dig_reset ( 
                 //Inputs
                 RESETN,
                 MR_RESET_TO_RCB,
                 AN_ACTIVE,
                 MR_POWER_DOWN,
                 MR_100MBS,
                 CLKXTAL,
                 BASE100TX_DIS_ER,
                 ANLG_TST,
                 MR_ERR_THRESH,
                 MR_ERR_TIMER,
                 MR_MSE_NORESET,
                 CLKPLL_IN,
                 txc25_enable,
                 RX100_STABLE,
                 RX100_ER,
                 TP_DISC,
                 RXCLK125,
                 XOFF_20,
                 MR_FAST_MSE,
                 IDLE_DSP_RESET,

                 //Outputs
                 RESETREG,
                 RESETPLL,
                 RESET25,
                 RESET_DSP,
                 RESET_DSP25,
                 RESETRX125,
                 MR_RESET_DONE,
                 RPD_10_DISB,
                 RPD_100_DISB 
                 );

//
// I/O Declarations
//
input           RESETN;          // Master reset
input           MR_RESET_TO_RCB; // Programmable reset for all digital logic
input           AN_ACTIVE;       // Indicates auto-negotiate enabled
input           MR_POWER_DOWN;   // Flags channel powered down
input           MR_100MBS;       // Flags mode is tx100
input           CLKXTAL;         // On-board crystal output (25 MHz)
input           BASE100TX_DIS_ER;   // 100 Base T disable
input           ANLG_TST;        // No soft reset in analog test mode
input           CLKPLL_IN;       // System clock 125MHz or 160MHz
input           txc25_enable;    // 25MHz clock enable
input [4:0]     MR_ERR_THRESH ;  // Error count threshold
input [9:0]     MR_ERR_TIMER ;   // Terminal count for error timer
input           MR_MSE_NORESET;  // Inhibit reset due to MSE time
input           RX100_STABLE;    // 100TX receive path stable
input           RX100_ER;        // Error Indicator
input           TP_DISC;         // Twisted pair disconnect, set for loopbacks
input           RXCLK125;        // Recovered clock used by DSP
input           XOFF_20;         // 20usec counter triggered by resets and mode changes
input           MR_FAST_MSE;     // Sets fast MSE_GOOD count from 10msecs to 1msec
input           IDLE_DSP_RESET;  // Resets DSP if <20 consecutive idles in 1.5msec

output          RESETPLL;        // Main reset for PCS and analog sections
output          RPD_10_DISB;     // Disables tx10 clocks generated by dig_clk
output          RPD_100_DISB;    // Disables tx100 clocks generated by dig_clk
output          RESET25;         // Generates xtal sync reset in dig_clk block
output          MR_RESET_DONE;   // Used for MDI read, low while in reset
output          RESETREG;        // Resets all control registers
output          RESET_DSP;       // DSP reset control
output          RESET_DSP25;     // DSP reset synced to CLKXTAL
output          RESETRX125;      // Reset for recovered clock RXCLK125

//
// I/O Type Declarations
//
wire            RESETN;         
wire            MR_RESET_TO_RCB;
wire            AN_ACTIVE;
wire            MR_POWER_DOWN;
wire            MR_FAST_MSE;
wire            MR_100MBS;
wire            CLKXTAL;
wire            BASE100TX_DIS_ER;
wire            ANLG_TST;
wire            CLKPLL_IN;       // System clock 125MHz or 160MHz
wire            txc25_enable;    // 25MHz clock enable
wire  [4:0]     MR_ERR_THRESH ;	
wire  [9:0]     MR_ERR_TIMER ;	
wire            MR_MSE_NORESET;
wire            RX100_STABLE;
wire            RX100_ER;
wire            TP_DISC;
wire            RXCLK125;
wire            XOFF_20;

reg  [1:0]      RESETPLL_SYN;
reg             RPD_10_DISB;
reg             RPD_100_DISB;
reg  [1:0]      RESET25_SYN;
reg             MR_RESET_DONE;
reg             RESETREG;
reg  [1:0]      RESET_DSP_SYN;
reg  [1:0]      RESET_DSP25_SYN;
reg  [1:0]      RESETRX125_SYN;

wire            RESETPLL;
wire            RESET25;
wire            RESET_DSP;
wire            RESET_DSP25;
wire            RESETRX125;

//
// Internal Signal Declarations
//
reg [1:0]       sync_RX100_ER;
reg [3:0]       countreset;            // Reset Time Up
reg [3:0]       countreset2;           // Error count reset
reg [9:0]       counterrors;           // Error Time counter
reg [4:0]       numerrors;             // Number of Errors
reg             reset_errors;          // counter resets
reg             RESET_MSE;             // counter resets
reg [17:0]      countmse;              // 10 msec counter
reg             RESET_DEGLITCH_XTAL1;  // 1st deglitching and sync for XTALCLK
reg             RESET_DEGLITCH_XTAL2;  // 2nd deglitching and sync for XTALCLK
reg             SOFT_RST_DEL;          // Delayed softreset from dig_regs
reg             SOFT_RST_DEL1;         // Delayed SOFT_RST_DEL
reg [2:0]       SW_RESET_COUNT;        // MR_RESET_DONE counter
reg [12:0]      COUNT_7500;            // counter

reg             prev1_an_active;       // Meta-stable for AN-ACTIVE.   
reg             prev2_an_active;       // Meta-stable for AN-ACTIVE.   
reg             prev3_an_active;       // Meta-stable for AN-ACTIVE.   

reg             meta_1_mrd_clkpll_in;  // Meta for MR_RESET_DONE, CLKPLL_IN.
reg             mrd_clkpll_in_synced;  // Meta for MR_RESET_DONE, CLKPLL_IN.
reg             meta_1_mrd_rxclk125;   // Meta for MR_RESET_DONE, RXCLK125.
reg             mrd_rxclk125_synced;   // Meta for MR_RESET_DONE, RXCLK125.

wire            inhibit_timer_reset;   // error timer is not to trip reset
wire            end_error_counters;
wire            RESET_DSP_UNSYNC;      // DSP reset before it's synced to RXCLK
wire            SW_RESET_START;        // Ssoft reset, starts SW_RESET_COUNTER
wire            RESET_DONE_CLEAR;      // Clears MR_RESET_DONE and SW_RESET_COUNTER
wire            RESET_7500;            // Deglitched hardware reset that starts
                                       // 7500 counter
wire            SW_HW_PWRDWN_RST;      // Set during a powerdown, 
                                       // hard or soft reset
wire [17:0]     MSE_GOOD_TIME;         // Time for MSE_GOOD being low before reset_dsp

//
// Parameter Declarations
//
`define  TRESET         4'h1		// 5 cycles of 125Mhz ~ 1 cycle of 25

`ifdef SPEEDUP
  `define  TCOUNT_7500    13'd0004	// 4 Cycles of CLKXTAL
  `define  TMSEGOOD       18'd25	// 1usec for MSE to be GOOD
  `define  TMSEGOOD_FAST  18'd25	// 1usec for MSE GOOD to speed up test
`else
  `define  TCOUNT_7500    13'd7500	// 7500 Cycles of CLKXTAL
  `define  TMSEGOOD       18'h3D090	// 10msec for MSE to be GOOD
  `define  TMSEGOOD_FAST  18'h061A8	// 1msec for MSE GOOD to speed up test
`endif
// For Top level simulation purposes only
initial #110 
  $display("TCOUNT = ", `TCOUNT_7500, "\n TMSEGOOD = ", `TMSEGOOD,
           "\nTMSEGOOD_FAST = ", `TMSEGOOD_FAST ); 

//------------------------------------------------------------------------------
// MAIN UNIT CODE
//------------------------------------------------------------------------------
//
   assign RESET_DONE_CLEAR    = (SW_RESET_COUNT == 3'd6);
   assign SW_RESET_START      = !SOFT_RST_DEL1 & MR_RESET_TO_RCB;
   assign inhibit_timer_reset = (MR_ERR_THRESH == 5'h00) | 
                                  (MR_ERR_TIMER == 10'h000);
   assign RESET_DSP_UNSYNC    = ANLG_TST ? RESETREG : (SW_HW_PWRDWN_RST | 
                                RESET_MSE | TP_DISC | IDLE_DSP_RESET |
                                BASE100TX_DIS_ER | XOFF_20 |
                                (reset_errors & !inhibit_timer_reset));

   assign SW_HW_PWRDWN_RST    = MR_RESET_TO_RCB | MR_POWER_DOWN;

   assign MSE_GOOD_TIME       = MR_FAST_MSE ? `TMSEGOOD_FAST : `TMSEGOOD;


//------------------------------------------------------------------------------         
// Deglitching registers for hard reset RESETN and syncing so 
// that reset will be deasserted synchronously to CLKXTAL
//------------------------------------------------------------------------------
//
   always@(negedge CLKXTAL)
      begin : p_deglitch
      RESET_DEGLITCH_XTAL1 <= !RESETN;
      RESET_DEGLITCH_XTAL2 <= RESET_DEGLITCH_XTAL1;
      end // p_deglitch

   // Generate active high deglitched hardware reset synced to XTALCLK
   assign RESET_7500 = (!RESETN & RESET_DEGLITCH_XTAL1 & RESET_DEGLITCH_XTAL2);

//------------------------------------------------------------------------------
// Generate reset thats asserted for 7500xtal clocks following
// a hard reset to allow the clock crystal to settle after powerup.
//------------------------------------------------------------------------------
//
   always@(negedge CLKXTAL or posedge RESET_7500)
      begin :p_COUNT_7500
      if (RESET_7500)
         COUNT_7500 <= `TCOUNT_7500;
      else if (|COUNT_7500)
         COUNT_7500 <= COUNT_7500 - 1'd1;
      else
         COUNT_7500 <= 13'd0;
      end // p_COUNT_7500

   always@(COUNT_7500)
      begin : p_RESETREG
      if (COUNT_7500 != 13'd0)
         RESETREG = 1'b1;
      else
         RESETREG = 1'b0;
      end // p_RESETREG

//------------------------------------------------------------------------------   
// MR_RESET_DONE must be asserted for long enough that CLKXTAL has stablised 
// and can sample it, thus resetting MR_RESET_TO_RCB.  
//------------------------------------------------------------------------------
//
   always@(posedge CLKXTAL or posedge RESETREG)
      begin : p_MR_RESET_DONE
      if (RESETREG)
         begin
         MR_RESET_DONE  <= 1'b1;
         SW_RESET_COUNT <= 3'd0;
         end
      else if (RESET_DONE_CLEAR) 
         begin
         MR_RESET_DONE  <= 1'b1;
         SW_RESET_COUNT <= 3'd0;
         end
      else if (SW_RESET_START)
         begin
         MR_RESET_DONE <= 1'b0;
         SW_RESET_COUNT <= SW_RESET_COUNT + 1;
         end
      else if (SW_RESET_COUNT > 3'd0)
         begin
         MR_RESET_DONE <= 1'b0;
         SW_RESET_COUNT <= SW_RESET_COUNT + 1;
         end
      else      
         begin
         MR_RESET_DONE <= 1'b1;
         SW_RESET_COUNT <= 3'd0;
         end
      end // p_MR_RESET_DONE

//------------------------------------------------------------------------------
// Sync SW_HW_PWRDWN_RST to 25MHz XTAL clock
//------------------------------------------------------------------------------
//
   always@(negedge CLKXTAL or posedge RESETREG)
      begin : p_RESET25_SYN
      if (RESETREG)
         RESET25_SYN <= 2'b11;
      else
         begin
         RESET25_SYN[0] <= SW_HW_PWRDWN_RST;
         RESET25_SYN[1] <= RESET25_SYN[0];
         end
      end //p_RESET25_SYN

   assign RESET25 = RESET25_SYN[1];


//------------------------------------------------------------------------------
// Stabilize MR_RESET_DONE for CLKPLL_IN
//------------------------------------------------------------------------------
   always@(posedge CLKPLL_IN or posedge RESETREG)
      begin : p_meta_xtal_clkpll  
      if (RESETREG)
         begin
         meta_1_mrd_clkpll_in <= 1'b0;
         mrd_clkpll_in_synced <= 1'b0;
         end
      else
         begin
         meta_1_mrd_clkpll_in <= MR_RESET_DONE;
         mrd_clkpll_in_synced <= meta_1_mrd_clkpll_in;
         end
      end   // p_meta_xtal_clkpll  

//------------------------------------------------------------------------------
// Sync SOFT_PWRDWN_RST to PLL clock
//------------------------------------------------------------------------------
//
   always@(negedge CLKPLL_IN or posedge RESETREG)
      begin : p_RESETPLL
      if (RESETREG)
         RESETPLL_SYN <= 2'b11;
      else if (!mrd_clkpll_in_synced)
         RESETPLL_SYN <= 2'b00;
      else
         begin
         RESETPLL_SYN[0] <= SW_HW_PWRDWN_RST;
         RESETPLL_SYN[1] <= RESETPLL_SYN[0];
         end
      end // p_RESETPLL_SYN

   assign RESETPLL = RESETPLL_SYN[1];

//------------------------------------------------------------------------------
// Stabilize MR_RESET_DONE for RXCLK125
//------------------------------------------------------------------------------
   always@(posedge RXCLK125 or posedge RESETREG)
      begin : p_meta_xtal_rxclk125  
      if (RESETREG)
         begin
         meta_1_mrd_rxclk125 <= 1'b0;
         mrd_rxclk125_synced <= 1'b0;
         end
      else
         begin
         meta_1_mrd_rxclk125 <= MR_RESET_DONE;
         mrd_rxclk125_synced <= meta_1_mrd_rxclk125;
         end
      end   // p_meta_xtal_rxclk125 
      
       
//------------------------------------------------------------------------------
// Sync SOFT_PWRDWN_RST to RXCLK125 clock
//------------------------------------------------------------------------------
//
   always@(negedge RXCLK125 or posedge RESETREG)
      begin : p_RESETRX125_SYN
      if (RESETREG)
         RESETRX125_SYN <= 2'b11;
      else if (!mrd_rxclk125_synced)
         RESETRX125_SYN <= 2'b00;
      else
         begin
         RESETRX125_SYN[0] <= SW_HW_PWRDWN_RST;
         RESETRX125_SYN[1] <= RESETRX125_SYN[0];
         end
      end // p_RESETRX125_SYN

   assign RESETRX125 = RESETRX125_SYN[1];

//------------------------------------------------------------------------------
// Generate a delayed version of MR_RESET to use for 
// generating RESET_DONE
//------------------------------------------------------------------------------
//
   always@(posedge CLKXTAL or posedge RESETREG)
      begin : p_SOFT_RST_DEL
      if (RESETREG)
          begin
            SOFT_RST_DEL <= 1'b0;
            SOFT_RST_DEL1 <= 1'b0;  // may be able to remove when CLKXTAL
          end                      // more robustly generated
      else   
          begin
            SOFT_RST_DEL <= MR_RESET_TO_RCB;
            SOFT_RST_DEL1 <= SOFT_RST_DEL;
          end
      end // p_SOFT_RST_DEL

//------------------------------------------------------------------------------
// Generate RESET_DSP syncronised to RCLK125
//------------------------------------------------------------------------------
//
   always@(negedge RXCLK125 or posedge RESETREG)
      begin : p_RESET_DSP_SYN
      if (RESETREG)
         RESET_DSP_SYN <= 2'b11;
      else
         begin
         RESET_DSP_SYN[0] <= RESET_DSP_UNSYNC;
         RESET_DSP_SYN[1] <= RESET_DSP_SYN[0];
         end         
      end // p_RESET_DSP_SYN

   assign RESET_DSP = RESET_DSP_SYN[1];

//------------------------------------------------------------------------------
// Generate RESET_DSP syncronised to XTAL25, required for RXSTABLE generation
//------------------------------------------------------------------------------
//
   always@(negedge CLKXTAL or posedge RESETREG)
      begin : p_RESET_DSP25_SYN
      if (RESETREG)
          RESET_DSP25_SYN <= 2'b11;
      else
         begin
         RESET_DSP25_SYN[0] <= RESET_DSP_UNSYNC;
         RESET_DSP25_SYN[1] <= RESET_DSP25_SYN[0];
         end
      end // p_RESET_DSP25_SYN

   assign RESET_DSP25 = RESET_DSP25_SYN[1];


//------------------------------------------------------------------------------
// Meta-stability delays for AN_ACTIVE input.
//------------------------------------------------------------------------------
//
   always@(posedge CLKXTAL or posedge RESETREG)
      begin : p_prev_an_active
         if (RESETREG)
         begin
            prev1_an_active <= 1'b0;
            prev2_an_active <= 1'b0;
            prev3_an_active <= 1'b0;
         end

         else
         begin
            prev1_an_active <= AN_ACTIVE;
            prev2_an_active <= prev1_an_active;
            prev3_an_active <= prev2_an_active;
         end
      end // p_prev_an_active


//------------------------------------------------------------------------------
// Generate RPD_10_DISB
//------------------------------------------------------------------------------
//
   always@(posedge CLKXTAL or posedge RESETREG)
      begin : p_RPD_10_DISB
      if (RESETREG)
         RPD_10_DISB <= 1'b1;
      else
         RPD_10_DISB <= (!prev3_an_active & MR_100MBS) | MR_POWER_DOWN;
      end // p_RPD_10_DISB

 
//------------------------------------------------------------------------------
// Generate RPD_100_DISB
//------------------------------------------------------------------------------
// 
   always@(posedge CLKXTAL or posedge RESETREG)
      begin : p_RPD_100_DISB
      if (RESETREG)
         RPD_100_DISB <= 1'b0;
      else
         RPD_100_DISB <= (!prev3_an_active & !MR_100MBS) | MR_POWER_DOWN;
      end // p_RPD_100_DISB

//------------------------------------------------------------------------------
// MSE GOOD - wait for 10msec if not up then reset DSP
//------------------------------------------------------------------------------
   always@(posedge CLKPLL_IN or posedge RESETPLL)
      begin : p_countmse
      if (RESETPLL)
         begin
         countmse   <= 18'b0;
         RESET_MSE  <= 1'b0;
         countreset <= 1'b0;
         end
      else if (txc25_enable)
         begin
         if (RX100_STABLE | RESET_MSE | MR_MSE_NORESET)
            countmse <= 18'b0;
         else
            countmse <= countmse+18'b1;

         if (countmse == MSE_GOOD_TIME)
            begin
            RESET_MSE  <= 1'b1;
            countreset <= 4'b0;
            end

         if (RESET_MSE)
            begin
            countreset <= countreset + 4'b1;
            if (countreset==`TRESET)
               RESET_MSE <= 1'b0;
            end
         end
      else
         begin
         countmse   <= countmse;
         RESET_MSE  <= RESET_MSE;
         countreset <= countreset;
         end
      end // p_countmse

//------------------------------------------------------------------------------
// Code Violations - After MSE is stable count the number of 
// errors during the next MR_ERR_TIMER cycles
// Reset the DSP if greater than MR_ERR_THRESH
//------------------------------------------------------------------------------
//
    // Sync RX100ER to TXC25
   always@(posedge CLKPLL_IN or posedge RESETPLL)
      begin : p_sync_RX100_ER
      if (RESETPLL)
         sync_RX100_ER = 2'b00;
      else if (txc25_enable)
         sync_RX100_ER = {sync_RX100_ER[0],RX100_ER};
      else
         sync_RX100_ER = sync_RX100_ER;
      end // p_sync_RX100_ER

   // Check the number of errors is below error threshold
   // within the programmed or timer count
   always @(posedge CLKPLL_IN or posedge RESETPLL)
      begin : p_counterrors
      if (RESETPLL)
         begin
         counterrors  <= 10'b0;
         numerrors    <=  5'b0;
         reset_errors <=  1'b0;
         countreset2  <=  4'b0;
         end
      else if (txc25_enable & (!RX100_STABLE))
         begin
         counterrors  <= 10'b0;
         numerrors    <=  5'b0;
         reset_errors <=  1'b0;
         countreset2  <=  4'b0;
         end
      else if (txc25_enable)
         begin 
         if (reset_errors)
            begin
            countreset2 <= countreset2 + 4'b1;
            if (countreset2==`TRESET)
               reset_errors <= 1'b0;
            end
         else if (end_error_counters)
            begin
            reset_errors <= (numerrors == MR_ERR_THRESH);
            countreset2  <=  4'b0;
            counterrors  <= 10'b0;
            numerrors    <=  5'b0;
            end
         else
            begin
            counterrors <= counterrors + 10'b1;
            if (sync_RX100_ER[1])
               numerrors <= numerrors + 5'b1;
            else
               numerrors <= numerrors;
            end
         end
      else
         begin
         counterrors  <= counterrors;
         numerrors    <= numerrors;
         reset_errors <= reset_errors;
         countreset2  <= countreset2;
         end
      end // p_counterrors

   assign end_error_counters = (counterrors == MR_ERR_TIMER) |
                               (numerrors == MR_ERR_THRESH);



endmodule
